@@ -1,7 +1,10 @@
+import uuid
+
 from python_agent.common import constants
-from python_agent.common.constants import DEFAULT_WORKSPACEPATH, TestSelectionStatus
+from python_agent.common.constants import DEFAULT_WORKSPACEPATH
 from python_agent.utils import to_str_obj
 
+
 class ScmConfigArgs(object):
     def __init__(self, scm_provider, scm_version, scm_base_url, scm_type):
         self.scmProvider = scm_provider
@@ -9,8 +12,19 @@ class ScmConfigArgs(object):
         self.scmBaseUrl = scm_base_url
         self.scmType = scm_type
 
+
 class ConfigData(object):
-    def __init__(self, token=None, customer_id=None, server=None, proxy=None, build_session_id=None, is_compress=True):
+    def __init__(
+        self,
+        token=None,
+        customer_id=None,
+        server=None,
+        proxy=None,
+        build_session_id=None,
+        is_compress=True,
+    ):
+        self.agentId = str(uuid.uuid4())
+        self.agentType = None
         self.token = token
         self.server = server
         self.proxy = proxy
@@ -43,10 +57,17 @@ class ConfigData(object):
         self.perTest = True
         self.interval = constants.INTERVAL_IN_MILLISECONDS
         self.intervalSeconds = constants.INTERVAL_IN_SECONDS
-        self.testSelection = {"enable": True, "interval": constants.TEST_RECOMMENDATION.interval_sec, "timeout": constants.TEST_RECOMMENDATION.timeout_sec}
+        self.testSelection = {
+            "enable": True,
+            "interval": constants.TEST_RECOMMENDATION.interval_sec,
+            "timeout": constants.TEST_RECOMMENDATION.timeout_sec,
+        }
         self.testSelectionStatus = None
         self.test_selection_enable = True
         self.isDisabled = False
+        self.testGroupId = None
+        self.auto_execution = True
+        self.drop_init_footprints = False
 
     def apply_scm_args(self, scm_args):
         """
@@ -66,8 +87,10 @@ class ConfigData(object):
                 self.scmType = scm_args.scmType
 
     def __str__(self):
-        return 'ConfigData:\n' + to_str_obj(self)
+        return "ConfigData:\n" + to_str_obj(self)
 
     def __eq__(self, other):
         return self.__dict__ == other.__dict__
 
+    def get_is_disabled(self):
+        return self.isDisabled
